# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import onfido.db.fields
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0003_auto_20161023_1455'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='event',
            name='onfido_id',
        ),
        migrations.RemoveField(
            model_name='event',
            name='status_after',
        ),
        migrations.RemoveField(
            model_name='event',
            name='status_before',
        ),
        migrations.AddField(
            model_name='event',
            name='raw',
            field=onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True),
        ),
        migrations.AddField(
            model_name='event',
            name='resource_id',
            field=models.CharField(default='', help_text='The Onfido id of the object that was updated, returned from the API.', max_length=40),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='event',
            name='status',
            field=models.CharField(default='', help_text='The status of the object after the event.', max_length=20),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='event',
            name='user',
            field=models.ForeignKey(related_name='onfido_events', blank=True, to=settings.AUTH_USER_MODEL, help_text='The user who triggered the event.', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
import datetime
import mock

from dateutil.parser import parse as date_parse

from django.contrib.auth.models import User
from django.db.models import Model, query
from django.test import TestCase
from django.utils.timezone import now as tz_now

from ..api import ApiError
from ..models import (
    BaseModel,
    BaseQuerySet,
    BaseStatusModel,
    Applicant,
    Check,
    Report,
    Event,
    CheckQuerySet
)


class TestBaseModel(BaseModel):

    class Meta:
        managed = False


class TestBaseStatusModel(BaseStatusModel):

    class Meta:
        managed = False


class BaseModelTests(TestCase):

    """onfido.models.BaseModel tests."""

    def test_defaults(self):
        obj = TestBaseModel()
        self.assertEqual(obj.id, None)
        self.assertEqual(obj.onfido_id, '')
        self.assertEqual(obj.created_at, None)
        self.assertEqual(obj.raw, {})

    @mock.patch.object(BaseModel, 'full_clean')
    @mock.patch.object(Model, 'save')
    def test_save(self, mock_save, mock_clean):
        """Test that save method returns self."""
        obj = TestBaseModel()
        self.assertEqual(obj.save(), obj)

    def test_href(self):
        """Test the href property."""
        raw = {'href': 'foo'}
        obj = TestBaseModel(raw=raw)
        self.assertEqual(obj.href, raw['href'])

    def test_parse(self):
        """Test the parse method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        obj = TestBaseModel().parse(data)
        self.assertEqual(obj.onfido_id, data['id'])
        self.assertEqual(obj.created_at, date_parse(data['created_at']))

    @mock.patch.object(BaseModel, 'save')
    @mock.patch('onfido.models.get')
    def test_fetch(self, mock_get, mock_save):
        """Test the fetch method calls the API."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
            "href": "/"
        }
        mock_get.return_value = data
        obj = TestBaseModel(raw={'href': '/'})
        obj.fetch()
        # check that it has update raw, but parsed the return value
        self.assertEqual(obj.raw, data)
        self.assertEqual(obj.onfido_id, data['id'])
        self.assertEqual(obj.created_at, date_parse(data['created_at']))
        # check that it has **not** called the save method
        mock_save.assert_not_called()

        # check what happens if href is missing
        obj = TestBaseModel(raw={})
        self.assertRaises(KeyError, obj.pull)

        # check what happens if API fails
        obj = TestBaseModel(raw={'href': '/'})
        response = mock.Mock()
        response.json.return_value = {
            "error": {
                "fields": {},
                "message": "Authorization error: please re-check your credentials",
                "type": "authorization_error"
            }
        }
        mock_get.side_effect = ApiError(response)
        self.assertRaises(ApiError, obj.pull)

    @mock.patch.object(BaseModel, 'save')
    @mock.patch.object(BaseModel, 'fetch')
    def test_pull(self, mock_fetch, mock_save):
        """Test the pull method calls fetch and save."""
        obj = TestBaseModel(raw={'href': '/'})
        mock_fetch.return_value = obj
        obj.pull()
        # check that it has parsed the return value
        mock_fetch.assert_called_once_with()
        mock_save.assert_called_once_with()


class BaseQuerySetTests(TestCase):

    """onfido.models.BaseQuerySet tests."""

    def create_applicant(self, username):
        """Create new Applicant and user."""
        data = {'id': username, 'created_at': tz_now().isoformat()}
        user = User.objects.create_user(username)
        applicant = Applicant.objects.create_applicant(user, raw=data)
        return applicant

    @mock.patch.object(BaseModel, 'fetch')
    def test_fetch(self, mock_fetch):
        """Test the queryset calls fetch on all objects."""
        # once applicant, one call
        self.create_applicant('fred')
        Applicant.objects.all().fetch()
        mock_fetch.assert_called_once_with()

        # add another applicant, should now call twice
        self.create_applicant('ginger')
        mock_fetch.reset_mock()
        Applicant.objects.all().fetch()
        self.assertEqual(mock_fetch.call_count, 2)

    @mock.patch.object(BaseModel, 'pull')
    def test_pull(self, mock_pull):
        """Test the queryset calls fetch on all objects."""
        # once applicant, one call
        self.create_applicant('fred')
        Applicant.objects.all().pull()
        mock_pull.assert_called_once_with()

        # add another applicant, should now call twice
        self.create_applicant('ginger')
        mock_pull.reset_mock()
        Applicant.objects.all().pull()
        self.assertEqual(mock_pull.call_count, 2)


class BaseStatusModelTests(TestCase):

    """onfido.models.BaseStatusModel tests."""

    def test_defaults(self):
        obj = TestBaseStatusModel()
        self.assertEqual(obj.id, None)
        self.assertEqual(obj.onfido_id, '')
        self.assertEqual(obj.created_at, None)
        self.assertEqual(obj.raw, {})
        self.assertEqual(obj.status, None)
        self.assertEqual(obj.result, None)
        self.assertEqual(obj.updated_at, None)
        self.assertEqual(obj.raw, {})

    def test_parse(self):
        """Test the parse method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        obj = TestBaseStatusModel().parse(data)
        self.assertEqual(obj.onfido_id, data['id'])
        self.assertEqual(obj.created_at, date_parse(data['created_at']))
        self.assertEqual(obj.status, data['status'])
        self.assertEqual(obj.result, data['result'])

    @mock.patch('onfido.signals.on_status_change.send')
    @mock.patch('onfido.signals.on_completion.send')
    @mock.patch.object(BaseStatusModel, 'pull')
    @mock.patch.object(BaseStatusModel, 'save')
    def test_update_status(self, mock_save, mock_pull, mock_complete, mock_update):
        """Test the update_status method."""
        now = datetime.datetime.now()

        def reset():
            mock_save.reset_mock()
            mock_pull.reset_mock()
            mock_complete.reset_mock()
            mock_update.reset_mock()
            event = Event(
                action='form.opened',
                status='after',
                onfido_id='foo',
                resource_type='check',
                completed_at=now
            )
            obj = TestBaseStatusModel(
                status='before'
            )
            assert obj.status == 'before'
            assert obj.updated_at is None
            return event, obj

        # try passing in something that is not a datetime
        event, obj = reset()
        event.completed_at = None
        self.assertRaises(AssertionError, obj.update_status, event)

        event, obj = reset()
        event.completed_at = now
        obj = obj.update_status(event)
        self.assertEqual(obj.status, event.status)
        self.assertEqual(obj.updated_at, now)
        mock_pull.assert_called_once_with()
        mock_save.assert_not_called()
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            event=event.action,
            status_before='before',
            status_after=event.status
        )
        mock_complete.assert_not_called()

        # if we send 'complete' as the status we should fire the second signal
        event, obj = reset()
        event.status = 'complete'
        obj = obj.update_status(event)
        self.assertEqual(obj.status, event.status)
        self.assertEqual(obj.updated_at, now)
        mock_pull.assert_called_once_with()
        mock_save.assert_not_called()
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            event=event.action,
            status_before='before',
            status_after=event.status
        )
        mock_complete.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj
        )

        # test that we can handle the API failing on pull()
        event, obj = reset()
        mock_pull.side_effect = Exception("Something went wrong in the API")
        obj = obj.update_status(event)
        self.assertEqual(obj.status, event.status)
        self.assertEqual(obj.updated_at, now)
        mock_pull.assert_called_once_with()
        mock_save.assert_called_once_with()
        mock_update.assert_called_once_with(
            TestBaseStatusModel,
            instance=obj,
            event=event.action,
            status_before='before',
            status_after=event.status
        )
        mock_complete.assert_not_called()

    @mock.patch('onfido.models.tz_now')
    def test__override_event(self, mock_now):
        """Test the _override_event method."""
        now = datetime.datetime.now()
        mock_now.return_value = now
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
            "href": "http://foo"
        }
        user = User()
        obj = TestBaseStatusModel().parse(data)
        event = obj._override_event(user)
        self.assertEqual(event.onfido_id, obj.onfido_id)
        self.assertEqual(event.action, "manual.override")
        self.assertEqual(event.resource_type, TestBaseStatusModel._meta.model_name)
        self.assertEqual(event.status, obj.status)
        self.assertEqual(event.completed_at, now)

    @mock.patch.object(Event, 'save')
    @mock.patch.object(BaseStatusModel, 'save')
    def test_mark_as_clear(self, mock_save, mock_event_save):
        """Test the mark_as_clear method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": None,
            "href": "http://foo"
        }
        user = User()
        obj = Check().parse(data)
        self.assertIsNone(obj.is_clear)
        obj = obj.mark_as_clear(user)
        mock_save.assert_called_once_with()
        mock_event_save.assert_called_once_with()
        self.assertTrue(obj.is_clear)

    @mock.patch.object(query.QuerySet, 'filter')
    def test_events(self, mock_filter):
        """Test the events method."""
        obj = TestBaseStatusModel(onfido_id='foo')
        obj.events()
        mock_filter.assert_called_once_with(
            onfido_id='foo',
            resource_type='testbasestatusmodel'
        )


class ApplicantManagerTests(TestCase):

    """onfido.models.ApplicantManager tests."""

    TEST_DATA = {
        "id": "14d2335e-4586-4ac4-aecd-b18296e7d675",
        "created_at": "2016-10-15T19:05:07Z",
    }

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(onfido_id='foo', user=self.user)

    @mock.patch.object(BaseModel, 'full_clean')
    def test_create_applicant(self, mock_clean):
        """Test the create method parses response."""
        data = ApplicantManagerTests.TEST_DATA
        applicant = Applicant.objects.create_applicant(user=self.user, raw=data)
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.raw, data)
        self.assertEqual(applicant.onfido_id, data['id'])
        self.assertEqual(applicant.created_at, date_parse(data['created_at']))


class ApplicantTests(TestCase):

    """onfido.model.Applicant tests."""

    def setUp(self):
        self.user = User(id=1, first_name=u"œ∑´®†¥")
        self.applicant = Applicant(onfido_id='foo', user=self.user)

    def test_defaults(self):
        """Test default property values."""
        applicant = self.applicant
        self.assertEqual(applicant.onfido_id, 'foo')
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.created_at, None)

    def test_save(self):
        """Test the save method."""
        self.user.save()
        applicant = self.applicant.save()
        self.assertEqual(applicant.onfido_id, 'foo')
        self.assertEqual(applicant.user, self.user)
        self.assertEqual(applicant.created_at, None)
        # test the related_name
        self.assertEqual(self.user.onfido_applicant, applicant)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        applicant = self.applicant
        self.assertIsNotNone(str(applicant))
        self.assertIsNotNone(unicode(applicant))
        self.assertIsNotNone(repr(applicant))


class CheckManagerTests(TestCase):

    """onfido.models.ApplicantManager tests."""

    @mock.patch.object(BaseModel, 'full_clean')
    def test_create_check(self, mock_clean):
        """Test the create method parses response."""
        user = User.objects.create_user(username='baz', first_name=u"œ∑´®†¥")
        applicant = Applicant(onfido_id='foo', user=user).save()
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        check = Check.objects.create_check(
            applicant=applicant,
            raw=data
        )
        self.assertEqual(check.user, user)
        self.assertEqual(check.applicant, applicant)
        self.assertEqual(check.onfido_id, data['id'])
        self.assertEqual(check.check_type, data['type'])
        self.assertEqual(check.status, data['status'])
        self.assertEqual(check.result, data['result'])
        self.assertEqual(check.created_at, date_parse(data['created_at']))


class CheckTests(TestCase):

    """onfido.models.Check tests."""

    def setUp(self):
        self.user = User.objects.create_user(username='fred', first_name=u"œ∑´®†¥")
        self.applicant = Applicant(
            onfido_id='foo',
            user=self.user
        ).save()
        self.check = Check(
            onfido_id='bar', user=self.user,
            applicant=self.applicant, check_type='standard'
        ).save()

    def test_defaults(self):
        """Test default property values."""
        check = Check()
        self.assertEqual(check.onfido_id, '')
        self.assertEqual(check.created_at, None)
        self.assertEqual(check.status, None)
        self.assertEqual(check.result, None)
        self.assertEqual(check.check_type, '')

    def test_save(self):
        """Test save method."""
        check = self.check
        self.assertEqual(check.onfido_id, 'bar')
        self.assertEqual(check.created_at, None)
        self.assertEqual(check.status, None)
        self.assertEqual(check.result, None)
        self.assertEqual(check.check_type, 'standard')
        # test the related_names
        self.assertEqual(self.applicant.checks.get(), check)
        self.assertEqual(self.user.onfido_checks.get(), check)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        check = self.check
        self.assertIsNotNone(str(check))
        self.assertIsNotNone(unicode(check))
        self.assertIsNotNone(repr(check))

    def test_parse(self):
        """Test the parse_raw method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "type": "standard",
            "result": "clear",
        }
        check = Check().parse(data)
        # real data taken from check.json
        self.assertEqual(check.onfido_id, "c26f22d5-4903-401f-8a48-7b0211d03c1f")
        self.assertEqual(check.created_at, date_parse(data['created_at']))
        self.assertEqual(check.status, "awaiting_applicant")
        self.assertEqual(check.result, "clear")


class ReportManagerTests(TestCase):

    """onfido.models.ReportManager tests."""

    TEST_DATA = {
        "created_at": "2016-10-18T16:02:08Z",
        "id": "1ffd3e8a-da71-4674-a245-8b52f1492191",
        "name": "identity",
        "result": "clear",
        "status": "awaiting_data",
        "variant": "standard"
    }

    def setUp(self):
        self.user = User.objects.create_user(username="foo", first_name=u"œ∑´®†¥")
        self.applicant = Applicant(
            user=self.user,
            onfido_id='foo',
        ).save()
        self.check = Check(
            user=self.user,
            applicant=self.applicant,
            check_type='standard',
            onfido_id='bar'
        ).save()

    @mock.patch.object(BaseModel, 'full_clean')
    def test_create_report(self, mock_clean):
        """Test the create method parses response."""
        data = ReportManagerTests.TEST_DATA
        report = Report.objects.create_report(
            check=self.check,
            raw=data
        )
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.onfido_id, data['id'])
        self.assertEqual(report.report_type, data['name'])
        self.assertEqual(report.status, data['status'])
        self.assertEqual(report.result, data['result'])
        self.assertEqual(report.created_at, date_parse(data['created_at']))


class ReportTests(TestCase):

    """onfido.models.Report tests."""

    def setUp(self):
        self.user = User.objects.create_user(
            username="foo",
            first_name=u"œ∑´®†¥"
        )
        self.applicant = Applicant(
            onfido_id='foo',
            user=self.user
        ).save()
        self.check = Check(
            onfido_id='bar',
            user=self.user,
            applicant=self.applicant,
            check_type='standard'
        ).save()
        self.report = Report(
            user=self.user, onfido_id='foo',
            onfido_check=self.check, report_type='identity'
        )

    def test_defaults(self):
        """Test default property values."""
        report = self.report
        self.assertEqual(report.onfido_id, 'foo')
        self.assertEqual(report.created_at, None)
        self.assertEqual(report.status, None)
        self.assertEqual(report.result, None)
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.report_type, 'identity')

    def test_save(self):
        """Test save method."""
        report = self.report.save()
        self.assertEqual(report.onfido_id, 'foo')
        self.assertEqual(report.created_at, None)
        self.assertEqual(report.status, None)
        self.assertEqual(report.result, None)
        self.assertEqual(report.user, self.user)
        self.assertEqual(report.onfido_check, self.check)
        self.assertEqual(report.report_type, 'identity')
        # test the related_names
        self.assertEqual(self.check.reports.get(), report)
        self.assertEqual(self.user.onfido_reports.get(), report)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        report = self.report
        self.assertIsNotNone(str(report))
        self.assertIsNotNone(unicode(report))
        self.assertIsNotNone(repr(report))

    def test_parse(self):
        """Test the parse_raw method."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "result": "clear",
            "name": "identity",
        }
        report = Report().parse(data)
        # real data taken from check.json
        self.assertEqual(report.onfido_id, "c26f22d5-4903-401f-8a48-7b0211d03c1f")
        self.assertEqual(report.created_at, date_parse(data['created_at']))
        self.assertEqual(report.status, "awaiting_applicant")
        self.assertEqual(report.result, "clear")
        self.assertEqual(report.report_type, "identity")

    def test_parse_scrubs_data(self):
        """Test the parse method also removes sensitive data."""
        data = {
            "id": "c26f22d5-4903-401f-8a48-7b0211d03c1f",
            "created_at": "2016-10-15T19:05:50Z",
            "status": "awaiting_applicant",
            "result": "clear",
            "name": "identity",
            "breakdown": {},
            "properties": {}
        }
        report = Report().parse(data)
        # the default report scrubber should have removed data:
        self.assertFalse('breakdown' in report.raw)
        self.assertFalse('properties' in report.raw)


class EventTests(TestCase):

    """onfido.models.Event tests."""

    TEST_DATA = {
        "payload": {
            "resource_type": "check",
            "action": "check.form_opened",
            "object": {
                "id": "6d7ee353-db1e-4b45-9034-f7e75198cbe0",
                "status": "awaiting_applicant",
                "completed_at": "2016-10-23 12:52:33 UTC",
                "href": "https://api.onfido.com/v1/applicants/xxx"
            }
        }
    }

    def setUp(self):
        self.user = User.objects.create_user(
            username="foo",
            first_name=u"œ∑´®†¥"
        )
        self.applicant = Applicant(
            onfido_id='foo',
            user=self.user
        ).save()
        self.check = Check(
            onfido_id='bar',
            user=self.user,
            applicant=self.applicant,
            check_type='standard'
        ).save()

    def test__resource_manager(self):
        """Test the _resource_manager method."""
        event = Event()
        self.assertRaises(AssertionError, event._resource_manager)
        event.resource_type = 'foo'
        self.assertRaises(AssertionError, event._resource_manager)

        event.resource_type = 'check'
        self.assertEqual(event._resource_manager(), Check.objects)
        event.resource_type = 'report'
        self.assertEqual(event._resource_manager(), Report.objects)

    @mock.patch.object(CheckQuerySet, 'get')
    def test_resource(self, mock_get):
        """Test the resource property."""
        event = Event().parse(EventTests.TEST_DATA)
        event.resource
        mock_get.assert_called_once_with(onfido_id=event.onfido_id)

    @mock.patch.object(CheckQuerySet, 'get')
    def test_user(self, mock_get):
        """Test the resource property."""
        event = Event().parse(EventTests.TEST_DATA)
        event.user
        mock_get.assert_called_once_with(onfido_id=event.onfido_id)

    def test_defaults(self):
        """Test default property values."""
        event = Event()
        # real data taken from check.json
        self.assertEqual(event.resource_type, '')
        self.assertEqual(event.onfido_id, '')
        self.assertEqual(event.action, '')
        self.assertEqual(event.status, '')
        self.assertEqual(event.completed_at, None)
        self.assertEqual(event.raw, {})

    def test_save(self):
        """Test save method."""
        data = EventTests.TEST_DATA
        event = Event().parse(data).save()
        # real data taken from check.json
        self.assertEqual(event.resource_type, data['payload']['resource_type'])
        self.assertEqual(event.onfido_id, data['payload']['object']['id'])
        self.assertEqual(event.action, data['payload']['action'])
        self.assertEqual(event.status, data['payload']['object']['status'])
        self.assertEqual(event.completed_at, date_parse(data['payload']['object']['completed_at']))
        self.assertEqual(event.raw, data)

    def test_unicode_str_repr(self):
        """Test string representations handle unicode."""
        event = Event().parse(EventTests.TEST_DATA)
        self.assertIsNotNone(str(event))
        self.assertIsNotNone(unicode(event))
        self.assertIsNotNone(repr(event))

    def test_parse(self):
        """Test the parse_raw method."""
        data = EventTests.TEST_DATA
        event = Event().parse(data)
        # real data taken from check.json
        self.assertEqual(event.resource_type, data['payload']['resource_type'])
        self.assertEqual(event.onfido_id, data['payload']['object']['id'])
        self.assertEqual(event.action, data['payload']['action'])
        self.assertEqual(event.status, data['payload']['object']['status'])
        self.assertEqual(event.completed_at, date_parse(data['payload']['object']['completed_at']))
        self.assertEqual(event.raw, data)



# -*- coding: utf-8 -*-
import mock

from django.test import TestCase

from .. import settings


class SettingsTests(TestCase):

    """onfido.settings module tests."""

    def test_defaults(self):
        """Confirm the default settings exist."""
        self.assertEqual(settings.API_ROOT, "https://api.onfido.com/v2/")
        self.assertEqual(settings.API_KEY, None)
        self.assertEqual(settings.LOG_EVENTS, True)
        self.assertEqual(settings.WEBHOOK_TOKEN, None)
        self.assertEqual(settings.TEST_MODE, False)

    def test_default_report_scrubber(self):
        """Test the report_scrubber default function."""
        data = {
            'foo': 'bar',
            'breakdown': {},
            'properties': {}
        }
        # default function should remove breakdown and properties
        data = settings.DEFAULT_REPORT_SCRUBBER(data)
        self.assertFalse('breakdown' in data)
        self.assertFalse('properties' in data)

    # mock scrubber that does nothing and returns the data unchanged
    @mock.patch('onfido.settings.scrub_report_data', lambda d: d)
    def test_override_report_scrubber(self):
        """Test the report_scrubber default function."""
        data = {
            'foo': 'bar',
            'breakdown': {},
            'properties': {}
        }
        # import here otherwise the mock is ineffective
        from ..settings import scrub_report_data
        # default function should remove breakdown and properties
        scrub_report_data(data)
        self.assertTrue('breakdown' in data)
        self.assertTrue('properties' in data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0005_remove_event_user'),
    ]

    operations = [
        migrations.RenameField(
            model_name='event',
            old_name='completed_at',
            new_name='created_at',
        ),
        migrations.RenameField(
            model_name='event',
            old_name='resource_id',
            new_name='onfido_id'
        )
    ]



# -*- coding: utf-8 -*-
"""
onfido signals.

Signals are fired when corresponding callbacks are received.

See https://documentation.onfido.com/#webhooks

"""
from django.dispatch import Signal

# fired after the status of a report /check is updated
# e.g. (obj, 'report.completed', 'pending', 'complete')
on_status_change = Signal(
    providing_args=['instance', 'event', 'status_before', 'status_after']
)

# 'complete' is the terminal state of both checks and reports, and as
# such it gets special treatment - it's the event that most people will
# be interested, so instead of having to filter out the on_status_change
# signal that results in completion, we have a dedicated signal.
on_completion = Signal(
    providing_args=['instance', 'completed_at']
)



# This file was sourced from github.com/django-extensions/django-extensions

# but split off into this project to allow patching for Django 1.3+ to code
# around DeprecationWarnings
import datetime
from decimal import Decimal

import simplejson as json
# While Django 1.6+ recommends json over simplejson, we're still using it
# here because simplejson gets Decimal reincantation right (ie, it does it),
# whereas stdlib's json returns the value as a plain string.

from django.db import models
from django.conf import settings
from django.utils.encoding import force_text


"""
JSONField automatically serializes most Python terms to JSON data.
Creates a TEXT field with a default value of "{}".

 from django.db import models
 from yunojuno.apps.core.fields import JSONField

 class LOL(models.Model):
     extra = JSONField()
"""


class JSONEncoder(json.JSONEncoder):
    # Note that we store all dates and datetimes as UTC, in the isoformat
    def default(self, obj):
        if isinstance(obj, Decimal):
            return str(obj)
        elif isinstance(obj, datetime.datetime):
            return obj.isoformat()
        elif isinstance(obj, datetime.date):
            return obj.isoformat()

        return json.JSONEncoder.default(self, obj)


def dumps(value):
    try:
        assert isinstance(value, dict)
    except AssertionError:
        try:
            assert isinstance(value, list)
        except AssertionError:
            raise

    return JSONEncoder().encode(value)


def loads(txt):
    value = json.loads(
        txt,
        parse_float=Decimal,
        encoding=settings.DEFAULT_CHARSET
    )
    try:
        assert isinstance(value, dict)
    except AssertionError:
        try:
            assert isinstance(value, list)
        except AssertionError:
            raise
    return value


class JSONDict(dict):
    """
    Hack so repr() called by dumpdata will output JSON instead of
    Python formatted data.  This way fixtures will work!
    """
    def __repr__(self):
        return dumps(self)


class JSONList(list):
    """
    Similar to JSONDict, but for Lists/Arrays
    """
    def __repr__(self):
        return dumps(self)


class JSONField(models.TextField):
    """JSONField is a generic textfield that neatly serializes/unserializes
    JSON objects seamlessly.  Main thingy must be a dict or a list (ie: JS map or array) """

    # Used so to_python() is called
    __metaclass__ = models.SubfieldBase

    def __init__(self, *args, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = '{}'
        models.TextField.__init__(self, *args, **kwargs)

    def to_python(self, value):
        """Convert our string value to JSON after we load it from the DB"""
        if not value:
            return {}
        elif isinstance(value, basestring):
            res = loads(value)
            try:
                assert isinstance(res, dict)
                return JSONDict(**res)
            except AssertionError:
                try:
                    assert isinstance(res, list)
                    return JSONList(res)
                except AssertionError:
                    raise
        else:
            return value

    def get_db_prep_save(self, value, connection):
        """Convert our JSON object to a string before we save"""

        if not isinstance(value, (list, dict)):
            # NOTE: this is a little ruthless, but works
            return super(JSONField, self).get_db_prep_save(
                "",
                connection
            )
        else:
            return super(JSONField, self).get_db_prep_save(
                dumps(value),
                connection
            )

    def deconstruct(self):
        """
        Returns enough information to recreate the field as a 4-tuple:
         * The name of the field on the model, if contribute_to_class has been run
         * The import path of the field, including the class: django.db.models.IntegerField
           This should be the most portable version, so less specific may be better.
         * A list of positional arguments
         * A dict of keyword arguments
        """

        name, path, args, kwargs = super(models.TextField, self).deconstruct()
        path = "onfido.db.fields.JSONField"
        # see https://docs.djangoproject.com/en/dev/howto/
        #   custom-model-fields/#custom-field-deconstruct-method

        # only include the default if it's not in the kwargs
        if 'default' not in kwargs:
            kwargs['default'] = '{}'

        return (
            force_text(self.name, strings_only=True),
            path,
            args,
            kwargs,
        )



# -*- coding: utf-8 -*-
import simplejson as json  # simplejson supports Decimal

from django.contrib import admin
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from .models import (
    Applicant,
    Check,
    Report,
    Event
)


class ResultMixin(object):

    """Adds custom action for overriding is_clear."""

    def mark_as_clear(self, request, queryset):
        """Call mark_as_clear on all objects in the queryset."""
        for obj in queryset:
            obj.mark_as_clear(request.user)

    mark_as_clear.short_description = _("Mark selected items as clear")


class EventsMixin(object):

    """Pretty print Events relating to an object."""

    def _events(self, obj):
        """Pretty print object events."""
        events = obj.events()
        html = ''.join(['<li>{}: {}</li>'.format(e.completed_at.date(), e.action) for e in events])
        return mark_safe("<ul>{}</ul>".format(html))

    _events.short_description = _("Related events")


class RawMixin(object):

    """Admin mixin used to pprint raw JSON fields."""

    def _raw(self, obj):
        """
        Return an indented HTML pretty-print version of JSON.

        Take the event_payload JSON, indent it, order the keys and then
        present it as a <code> block. That's about as good as we can get
        until someone builds a custom syntax function.

        """
        pretty = json.dumps(
            obj.raw,
            sort_keys=True,
            indent=4,
            separators=(',', ': ')
        )
        html = pretty.replace(" ", "&nbsp;").replace("\n", "<br>")
        return mark_safe("<code>{}</code>".format(html))

    _raw.short_description = _("Raw (from API)")


class UserMixin(object):

    """Admin mixin used to add _user function."""

    def _user(self, obj):
        """
        Return user's real name.

        It's impossible to create an Applicant in the Onfido API
        with a blank first/last name, so assuming that get_full_name
        will not be blank.

        """
        return obj.user.get_full_name().title()

    _user.short_description = 'User'


class ApplicantAdmin(RawMixin, UserMixin, admin.ModelAdmin):

    """Admin model for Applicant objects."""

    list_display = (
        'onfido_id', '_user', 'created_at'
    )
    list_filter = ('created_at',)
    ordering = (
        'user__first_name', 'user__last_name', 'user__username'
    )
    readonly_fields = (
        'onfido_id', 'user', 'created_at', '_raw'
    )
    search_fields = (
        'onfido_id', 'user__first_name', 'user__last_name'
    )
    raw_id_fields = ('user',)
    exclude = ('raw',)

admin.site.register(Applicant, ApplicantAdmin)


class CheckAdmin(ResultMixin, EventsMixin, RawMixin, UserMixin, admin.ModelAdmin):

    """Admin model for Check objects."""

    list_display = (
        'onfido_id', '_user', 'check_type', 'status',
        'result', 'created_at', 'updated_at', 'is_clear'
    )
    readonly_fields = (
        'onfido_id', 'user', 'created_at', 'applicant', 'check_type',
        'status', 'result', 'updated_at', '_raw', '_events'
    )
    search_fields = (
        'onfido_id', 'user__first_name', 'user__last_name'
    )
    list_filter = (
        'created_at', 'updated_at', 'check_type',
        'status', 'result'
    )
    ordering = (
        'user__first_name', 'user__last_name'
    )
    raw_id_fields = (
        'applicant', 'user'
    )
    exclude = ('raw',)
    actions = ('mark_as_clear',)

admin.site.register(Check, CheckAdmin)


class ReportAdmin(ResultMixin, EventsMixin, RawMixin, UserMixin, admin.ModelAdmin):

    """Admin model for Report objects."""

    list_display = (
        'onfido_id', '_user', 'report_type',
        'status', 'result', 'created_at', 'updated_at', 'is_clear'
    )
    ordering = (
        'user__first_name', 'user__last_name'
    )
    readonly_fields = (
        'onfido_id', 'user', 'onfido_check', 'report_type',
        'status', 'result', 'created_at', 'updated_at', '_raw'
    )
    search_fields = (
        'onfido_id', 'onfido_check__onfido_id',
        'user__first_name', 'user__last_name'
    )
    list_filter = (
        'created_at', 'updated_at', 'report_type', 'status', 'result'
    )
    raw_id_fields = (
        'onfido_check', 'user'
    )
    exclude = ('raw',)
    actions = ('mark_as_clear',)

admin.site.register(Report, ReportAdmin)


class EventAdmin(RawMixin, UserMixin, admin.ModelAdmin):

    """Admin model for Event objects."""

    list_display = (
        'onfido_id', 'resource_type', '_user', 'action',
        'status', 'completed_at'
    )
    list_filter = (
        'action', 'resource_type', 'status', 'completed_at'
    )
    readonly_fields = (
        'onfido_id', 'resource_type', 'action',
        'status', 'completed_at', '_raw'
    )
    search_fields = ('onfido_id',)
    exclude = ('raw',)

admin.site.register(Event, EventAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('onfido', '0002_event'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='action',
            field=models.CharField(help_text='The event name as returned from the API callback.', max_length=20),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('onfido', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('resource_type', models.CharField(help_text='The resource_type returned from the API callback.', max_length=20)),
                ('onfido_id', models.CharField(help_text='The id returned from the Onfido API.', max_length=40)),
                ('action', models.CharField(help_text='The resource_type returned from the API callback.', max_length=20)),
                ('status_before', models.CharField(help_text='The status of the object before the event.', max_length=20)),
                ('status_after', models.CharField(help_text='The status of the object after the event.', max_length=20)),
                ('completed_at', models.DateTimeField(help_text='The completed_at timestamp returned from the API callback.')),
                ('user', models.ForeignKey(related_name='onfido_events', to=settings.AUTH_USER_MODEL, help_text='The user who triggered the event.')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import onfido.db.fields
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Applicant',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('onfido_id', models.CharField(help_text='The id returned from the Onfido API.', unique=True, max_length=40)),
                ('created_at', models.DateTimeField(help_text='The timestamp returned from the Onfido API.', null=True, blank=True)),
                ('raw', onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True)),
                ('user', models.OneToOneField(related_name='onfido_applicant', to=settings.AUTH_USER_MODEL, help_text='Django user that maps to this applicant.')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Check',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('onfido_id', models.CharField(help_text='The id returned from the Onfido API.', unique=True, max_length=40)),
                ('created_at', models.DateTimeField(help_text='The timestamp returned from the Onfido API.', null=True, blank=True)),
                ('raw', onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True)),
                ('status', models.CharField(blank=True, max_length=20, null=True, help_text='The current state of the check / report (from API).', choices=[(b'Check', ((b'in_progress', b'In progress'), (b'awaiting_applicant', b'Awaiting applicant'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'reopened', b'Reopened'))), (b'Report', ((b'awaiting_data', b'Awaiting data'), (b'awaiting_approval', b'Awaiting approval'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'cancelled', b'Cancelled')))])),
                ('result', models.CharField(blank=True, max_length=20, null=True, help_text='The final result of the check / reports (from API).', choices=[(b'Check', ((b'clear', b'Clear'), (b'consider', b'Consider'))), (b'Report', ((b'clear', b'Clear'), (b'consider', b'Consider'), (b'unidentified', b'Unidentified')))])),
                ('updated_at', models.DateTimeField(help_text='The timestamp of the most recent status change (from API).', null=True, blank=True)),
                ('check_type', models.CharField(help_text='See https://documentation.onfido.com/#check-types', max_length=10, choices=[(b'express', b'Express check'), (b'standard', b'Standard check')])),
                ('applicant', models.ForeignKey(related_name='checks', to='onfido.Applicant', help_text='The applicant for whom the check is being made.')),
                ('user', models.ForeignKey(related_name='onfido_checks', to=settings.AUTH_USER_MODEL, help_text='The Django user (denormalised from Applicant to make navigation easier).')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Report',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('onfido_id', models.CharField(help_text='The id returned from the Onfido API.', unique=True, max_length=40)),
                ('created_at', models.DateTimeField(help_text='The timestamp returned from the Onfido API.', null=True, blank=True)),
                ('raw', onfido.db.fields.JSONField(default=b'{}', help_text='The raw JSON returned from the API.', null=True, blank=True)),
                ('status', models.CharField(blank=True, max_length=20, null=True, help_text='The current state of the check / report (from API).', choices=[(b'Check', ((b'in_progress', b'In progress'), (b'awaiting_applicant', b'Awaiting applicant'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'reopened', b'Reopened'))), (b'Report', ((b'awaiting_data', b'Awaiting data'), (b'awaiting_approval', b'Awaiting approval'), (b'complete', b'Complete'), (b'withdrawn', b'Withdrawn'), (b'paused', b'Paused'), (b'cancelled', b'Cancelled')))])),
                ('result', models.CharField(blank=True, max_length=20, null=True, help_text='The final result of the check / reports (from API).', choices=[(b'Check', ((b'clear', b'Clear'), (b'consider', b'Consider'))), (b'Report', ((b'clear', b'Clear'), (b'consider', b'Consider'), (b'unidentified', b'Unidentified')))])),
                ('updated_at', models.DateTimeField(help_text='The timestamp of the most recent status change (from API).', null=True, blank=True)),
                ('report_type', models.CharField(help_text='The name of the report - see https://documentation.onfido.com/#reports', max_length=20, choices=[(b'identity', b'Identity report'), (b'document', b'Document report'), (b'street_level', b'Street level report'), (b'facial_similarity', b'Facial similarity report'), (b'credit', b'Credit report'), (b'criminal_history', b'Criminal history'), (b'right_to_work', b'Right to work'), (b'ssn_trace', b'SSN trace')])),
                ('onfido_check', models.ForeignKey(related_name='reports', to='onfido.Check', help_text='Check to which this report is attached.')),
                ('user', models.ForeignKey(related_name='onfido_reports', to=settings.AUTH_USER_MODEL, help_text='The Django user (denormalised from Applicant to make navigation easier).')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Helper functions
from .api import post
from .models import (
    Applicant,
    Check,
    Report
)


def create_applicant(user, **kwargs):
    """Create an applicant in the Onfido system.

    Args:
        user: an auth.User instance to register as an applicant.
    Kwargs:
       any kwargs passed in are merged into the data dict sent to the API. This
       enables support for additional applicant properties - e.g dob, gender,
       country, and any others that may change over time.
       See https://documentation.onfido.com/#create-applicant for details.
    """
    data = {
        "first_name": user.first_name,
        "last_name": user.last_name,
        "email": user.email
    }
    data.update(kwargs)
    response = post('applicants', data)
    return Applicant.objects.create_applicant(user, response)


def create_check(applicant, check_type, reports, **kwargs):
    """
    Create a new Check (and child Reports).

    Args:
        applicant: Applicant for whom the checks are being made.
        check_type: string, currently only 'standard' is supported.
        reports: list of strings, each of which is a valid report type.

    Kwargs:
        any kwargs passed in are merged into the data dict sent to the API. This
        enables support for additional check properties - e.g. redirect_uri,
        tags, suppress_form_emails and any other that may change over time. See
        https://documentation.onfido.com/#checks for details.

    Returns a new Check object, and creates the child Report objects.

    """
    assert check_type == 'standard', (
        "Invalid check_type '{}', currently only 'standard' "
        "checks are supported.".format(check_type)
    )
    assert isinstance(reports, (list, tuple)), (
        "Invalid reports arg '{}', must be a list or tuple "
        "if supplied.".format(reports)
    )
    data = {
        "type": check_type,
        "reports": [{'name': r} for r in reports],
    }
    # merge in the additional kwargs
    data.update(kwargs)
    response = post('applicants/{}/checks'.format(applicant.onfido_id), data)
    check = Check.objects.create_check(applicant=applicant, raw=response)
    for report in response['reports']:
        Report.objects.create_report(check=check, raw=report)
    return check



# -*- coding: utf-8 -*-
from os import path, pardir, chdir
from setuptools import setup, find_packages

README = open(path.join(path.dirname(__file__), 'README.rst')).read()
# requirements.txt must be included in MANIFEST.in and include_package_data must be True
# in order for this to work; ensures that tox can use the setup to enforce requirements
REQUIREMENTS = '\n'.join(open(path.join(path.dirname(__file__), 'requirements.txt')).readlines())
# allow setup.py to be run from any path
chdir(path.normpath(path.join(path.abspath(__file__), pardir)))

setup(
    name="django-onfido",
    version="0.8.1",
    packages=find_packages(),
    install_requires=REQUIREMENTS,
    include_package_data=True,
    description='Django app for integration with Onfido.',
    long_description=README,
    url='https://github.com/yunojuno/django-onfido',
    author='Hugo Rodger-Brown',
    author_email='code@yunojuno.com',
    maintainer='Hugo Rodger-Brown',
    maintainer_email='hugo@yunojuno.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
"""onfido views.

The Onfido views are mainly callback handlers for the updates
in status' of checks and reports.

See https://documentation.onfido.com/?shell#webhooks

"""
import json
import logging

from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt

from .decorators import verify_signature
from .models import Check, Report, Event
from .settings import LOG_EVENTS

logger = logging.getLogger(__name__)


@csrf_exempt
@verify_signature()
def status_update(request):
    """
    Handle event callbacks from the API.

    This is the request handler which does little other than
    log the request and call the _webhook function which does
    the processing. Done like this to make it easier to test
    without having to set up request objects.

    NB This view function will always return a 200 status -
    in order to prevent Onfido from endlessly retrying. The
    only exceptions to this are caused by the verify_signature
    decorator - if it cannot verify the callback, then it will
    return a 403 - which should be ok, as if Onfido sends the
    request it should never fail...

    """
    logger.debug("Received Onfido callback: {}".format(request.body))
    data = json.loads(request.body)
    try:
        event = Event()
        resource = event.parse(data).resource
        resource.update_status(event)
        if LOG_EVENTS:
            event.save()
        return HttpResponse("Update processed.")
    except KeyError as ex:
        logger.warn("Missing Onfido event content: %s", ex)
        return HttpResponse("Unexpected event content.")
    except AssertionError:
        logger.warn("Unknown Onfido resource type: %s", event.resource_type)
        return HttpResponse("Unknown resource type.")
    except Check.DoesNotExist:
        logger.warn("Onfido check does not exist: %s", event.onfido_id)
        return HttpResponse("Check not found.")
    except Report.DoesNotExist:
        logger.warn("Onfido report does not exist: %s", event.onfido_id)
        return HttpResponse("Report not found.")
    except Exception:
        logger.exception("Onfido update could not be processed.")
        return HttpResponse("Unknown error.")



# -*- coding: utf-8 -*-
"""onfido function decorators."""
from functools import wraps
import hashlib
import hmac
import logging

from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseForbidden

from .settings import WEBHOOK_TOKEN, TEST_MODE


logger = logging.getLogger(__name__)


def _hmac(token, text):
    """
    Calculate SHA1 HMAC digest from request body and token.

    Args:
        token: string, the webhook token from Onfido API settings.
        text: string, the text to hash.

    Return the SHA1 HMAC as a string.

    """
    auth_code = hmac.new(token, text, hashlib.sha1).hexdigest()
    logger.debug("Onfido callback request HMAC: %s", auth_code)
    return auth_code


def _match(token, request):
    """
    Calculate signature and return True if it matches header.

    Args:
        token: string, the webhook_token.
        request: an HttpRequest object from which the body content and
            X-Signature header will be extracted and matched.

    Returns True if there is a match.

    """
    try:
        signature = request.META['HTTP_X_SIGNATURE']
        logger.debug("Onfido callback X-Signature: %s", signature)
        logger.debug("Onfido callback request body: %s", request.body)
        return _hmac(token, request.body) == signature
    except KeyError:
        logger.warn("Onfido callback missing X-Signature - this may be an unauthorised request.")
        return False
    except Exception:
        return False


def verify_signature():
    """
    View function decorator used to verify Onfido webhook signatures.

    This function uses the WEBHOOK_TOKEN specified in settings to calculate
    the HMAC. If it doesn't exist, then this decorator will immediately fail
    hard with an ImproperlyConfigured exception.

    If TEST_MODE is on (ONFIDO_TEST_MODE setting) then the verification will
    be ignored.

    See: https://documentation.onfido.com/#webhooks

        Each webhook URL is associated with a secret token. This is
        visible in the response.token field when registering a webhook.

        Events sent to your application will be signed using this token:
        verifying the request signature on your server prevents attackers
        from imitating valid webhooks. The HMAC digest signature, generated
        using SHA-1, will be stored in a X-Signature header.

        When you receive an event, you should compute a hash using your
        secret token, and ensure that the X-Signature sent by Onfido
        matches that hash.

    If the HMAC signatures don't match, return a 403

    """
    def decorator(func):
        @wraps(func)
        def _wrapped_func(request, *args, **kwargs):
            if TEST_MODE:
                logger.debug("Ignoring Onfido callback verification (ONFIDO_TEST_MODE enabled)")
                return func(request, *args, **kwargs)
            if not WEBHOOK_TOKEN:
                raise ImproperlyConfigured("Missing ONFIDO_WEBHOOK_TOKEN")
            if _match(WEBHOOK_TOKEN, request):
                return func(request, *args, **kwargs)
            else:
                # logging as a warning means it'll likely appear in logs,
                # but it's by design - if people are sending invalid requests
                # we need to know.
                logger.warn("Onfido callback request verification failed.")
                return HttpResponseForbidden("Invalid X-Signature")
        return _wrapped_func
    return decorator
